@@ -3,7 +3,6 @@ import datetime
 import json
 import logging
 from pathlib import Path
-from typing import Optional, Union
 
 from mitmproxy import flowfilter
 from mitmproxy.http import HTTPFlow
@@ -67,7 +66,9 @@ class JSONUrlIndexWriter(UrlIndexWriter):
         res = flow.response
 
         if req is not None and res is not None:
-            urls = self.host_urls.setdefault(f"{req.scheme}://{req.host}:{req.port}", dict())
+            urls = self.host_urls.setdefault(
+                f"{req.scheme}://{req.host}:{req.port}", dict()
+            )
             methods = urls.setdefault(req.path, {})
             codes = methods.setdefault(req.method, set())
             codes.add(res.status_code)
@@ -88,8 +89,10 @@ class TextUrlIndexWriter(UrlIndexWriter):
         req = flow.request
         if res is not None and req is not None:
             with self.filepath.open("a+") as f:
-                f.write(f"{datetime.datetime.utcnow().isoformat()} STATUS: {res.status_code} METHOD: "
-                        f"{req.method} URL:{req.url}\n")
+                f.write(
+                    f"{datetime.datetime.utcnow().isoformat()} STATUS: {res.status_code} METHOD: "
+                    f"{req.method} URL:{req.url}\n"
+                )
 
     def save(self):
         pass
@@ -113,16 +116,21 @@ class UrlIndexAddon:
     The injection can be done using the URLInjection Add-on.
     """
 
-    index_filter: Optional[Union[str, flowfilter.TFilter]]
+    index_filter: str | flowfilter.TFilter | None
     writer: UrlIndexWriter
 
     OPT_FILEPATH = "URLINDEX_FILEPATH"
     OPT_APPEND = "URLINDEX_APPEND"
     OPT_INDEX_FILTER = "URLINDEX_FILTER"
 
-    def __init__(self, file_path: Union[str, Path], append: bool = True,
-                 index_filter: Union[str, flowfilter.TFilter] = filter_404, index_format: str = "json"):
-        """ Initializes the urlindex add-on.
+    def __init__(
+        self,
+        file_path: str | Path,
+        append: bool = True,
+        index_filter: str | flowfilter.TFilter = filter_404,
+        index_format: str = "json",
+    ):
+        """Initializes the urlindex add-on.
 
         Args:
             file_path: Path to file to which the URL index will be written. Can either be given as str or Path.
@@ -153,7 +161,7 @@ class UrlIndexAddon:
 
     def response(self, flow: HTTPFlow):
         """Checks if the response should be included in the URL based on the index_filter and adds it to the URL index
-            if appropriate.
+        if appropriate.
         """
         if isinstance(self.index_filter, str) or self.index_filter is None:
             raise ValueError("Invalid filter expression.")
