@@ -9,10 +9,13 @@ from unittest.mock import MagicMock
 
 import pytest
 
+from mitmproxy.addons.next_layer import _starts_like_quic
 from mitmproxy.addons.next_layer import NeedsMoreData
 from mitmproxy.addons.next_layer import NextLayer
 from mitmproxy.addons.next_layer import stack_match
+from mitmproxy.connection import Address
 from mitmproxy.connection import Client
+from mitmproxy.connection import TlsVersion
 from mitmproxy.connection import TransportProtocol
 from mitmproxy.proxy.context import Context
 from mitmproxy.proxy.layer import Layer
@@ -28,6 +31,7 @@ from mitmproxy.proxy.layers import TCPLayer
 from mitmproxy.proxy.layers import UDPLayer
 from mitmproxy.proxy.layers.http import HTTPMode
 from mitmproxy.proxy.layers.http import HttpStream
+from mitmproxy.proxy.layers.tls import HTTP1_ALPNS
 from mitmproxy.proxy.mode_specs import ProxyMode
 from mitmproxy.test import taddons
 
@@ -88,30 +92,92 @@ quic_client_hello = bytes.fromhex(
     "297c0013924e88248684fe8f2098326ce51aa6e5"
 )
 
+quic_short_header_packet = bytes.fromhex(
+    "52e23539dde270bb19f7a8b63b7bcf3cdacf7d3dc68a7e00318bfa2dac3bad12cb7d78112efb5bcb1ee8e0b347"
+    "641cccd2736577d0178b4c4c4e97a8e9e2af1d28502e58c4882223e70c4d5124c4b016855340e982c5c453d61d"
+    "7d0720be075fce3126de3f0d54dc059150e0f80f1a8db5e542eb03240b0a1db44a322fb4fd3c6f2e054b369e14"
+    "5a5ff925db617d187ec65a7f00d77651968e74c1a9ddc3c7fab57e8df821b07e103264244a3a03d17984e29933"
+)
+
 dns_query = bytes.fromhex("002a01000001000000000000076578616d706c6503636f6d0000010001")
 
+# Custom protocol with just base64-encoded messages
+# https://github.com/mitmproxy/mitmproxy/pull/7087
+custom_base64_proto = b"AAAAAAAAAAAAAAAAAAAAAA==\n"
+
+http_get = b"GET / HTTP/1.1\r\nHost: example.com\r\n\r\n"
+http_get_absolute = b"GET http://example.com/ HTTP/1.1\r\n\r\n"
+
+http_connect = b"CONNECT example.com:443 HTTP/1.1\r\nHost: example.com:443\r\n\r\n"
+
 
 class TestNextLayer:
-    def test_configure(self):
-        nl = NextLayer()
-        with taddons.context(nl) as tctx:
-            with pytest.raises(Exception, match="mutually exclusive"):
-                tctx.configure(
-                    nl, allow_hosts=["example.org"], ignore_hosts=["example.com"]
-                )
-
     @pytest.mark.parametrize(
         "ignore, allow, transport_protocol, server_address, data_client, result",
         [
             # ignore
             pytest.param(
-                [], [], "example.com", "tcp", b"", False, id="nothing ignored"
+                [], [], "tcp", "example.com", b"", False, id="nothing ignored"
             ),
             pytest.param(
                 ["example.com"], [], "tcp", "example.com", b"", True, id="address"
             ),
             pytest.param(
-                ["1.2.3.4"], [], "tcp", "example.com", b"", True, id="ip address"
+                ["192.0.2.1"], [], "tcp", "example.com", b"", True, id="ip address"
+            ),
+            pytest.param(
+                ["2001:db8::1"],
+                [],
+                "tcp",
+                "ipv6.example.com",
+                b"",
+                True,
+                id="ipv6 address",
+            ),
+            pytest.param(
+                ["example.com:443"],
+                [],
+                "tcp",
+                "example.com",
+                b"",
+                True,
+                id="port matches",
+            ),
+            pytest.param(
+                ["example.com:123"],
+                [],
+                "tcp",
+                "example.com",
+                b"",
+                False,
+                id="port does not match",
+            ),
+            pytest.param(
+                ["example.com"],
+                [],
+                "tcp",
+                "192.0.2.1",
+                http_get,
+                True,
+                id="http host header",
+            ),
+            pytest.param(
+                ["example.com"],
+                [],
+                "tcp",
+                "192.0.2.1",
+                http_get.replace(b"Host", b"X-Host"),
+                False,
+                id="http host header missing",
+            ),
+            pytest.param(
+                ["example.com"],
+                [],
+                "tcp",
+                "192.0.2.1",
+                http_get.split(b"\r\n", 1)[0],
+                NeedsMoreData,
+                id="incomplete http host header",
             ),
             pytest.param(
                 ["example.com"],
@@ -123,7 +189,13 @@ class TestNextLayer:
                 id="partial address match",
             ),
             pytest.param(
-                ["example.com"], [], "tcp", None, b"", False, id="no destination info"
+                ["example.com"],
+                [],
+                "tcp",
+                None,
+                b"",
+                False,
+                id="no destination info",
             ),
             pytest.param(
                 ["example.com"],
@@ -138,7 +210,7 @@ class TestNextLayer:
                 ["example.com"],
                 [],
                 "tcp",
-                None,
+                "192.0.2.1",
                 client_hello_with_extensions,
                 True,
                 id="sni",
@@ -147,7 +219,7 @@ class TestNextLayer:
                 ["example.com"],
                 [],
                 "tcp",
-                None,
+                "192.0.2.1",
                 client_hello_with_extensions[:-5],
                 NeedsMoreData,
                 id="incomplete client hello",
@@ -156,7 +228,7 @@ class TestNextLayer:
                 ["example.com"],
                 [],
                 "tcp",
-                None,
+                "192.0.2.1",
                 client_hello_no_extensions[:9] + b"\x00" * 200,
                 False,
                 id="invalid client hello",
@@ -174,7 +246,7 @@ class TestNextLayer:
                 ["example.com"],
                 [],
                 "udp",
-                None,
+                "192.0.2.1",
                 dtls_client_hello_with_extensions,
                 True,
                 id="dtls sni",
@@ -183,7 +255,7 @@ class TestNextLayer:
                 ["example.com"],
                 [],
                 "udp",
-                None,
+                "192.0.2.1",
                 dtls_client_hello_with_extensions[:-5],
                 NeedsMoreData,
                 id="incomplete dtls client hello",
@@ -192,20 +264,56 @@ class TestNextLayer:
                 ["example.com"],
                 [],
                 "udp",
-                None,
+                "192.0.2.1",
                 dtls_client_hello_with_extensions[:9] + b"\x00" * 200,
                 False,
                 id="invalid dtls client hello",
             ),
             pytest.param(
-                ["example.com"], [], "udp", None, quic_client_hello, True, id="quic sni"
+                ["example.com"],
+                [],
+                "udp",
+                "192.0.2.1",
+                quic_client_hello,
+                True,
+                id="quic sni",
             ),
             # allow
             pytest.param(
-                [], ["example.com"], "tcp", "example.com", b"", False, id="allow: allow"
+                [],
+                ["example.com"],
+                "tcp",
+                "example.com",
+                b"",
+                False,
+                id="allow: allow",
             ),
             pytest.param(
-                [], ["example.com"], "tcp", "example.org", b"", True, id="allow: ignore"
+                [],
+                ["example.com"],
+                "tcp",
+                "example.org",
+                b"",
+                True,
+                id="allow: ignore",
+            ),
+            pytest.param(
+                [],
+                ["example.com"],
+                "tcp",
+                "192.0.2.1",
+                client_hello_with_extensions,
+                False,
+                id="allow: sni",
+            ),
+            pytest.param(
+                [],
+                ["existing-sni.example"],
+                "tcp",
+                "192.0.2.1",
+                b"",
+                False,
+                id="allow: sni from parent layer",
             ),
             pytest.param(
                 [],
@@ -216,6 +324,25 @@ class TestNextLayer:
                 False,
                 id="allow: sni mismatch",
             ),
+            # allow with ignore
+            pytest.param(
+                ["binary.example.com"],
+                ["example.com"],
+                "tcp",
+                "example.com",
+                b"",
+                False,
+                id="allow+ignore: allowed and not ignored",
+            ),
+            pytest.param(
+                ["binary.example.com"],
+                ["example.com"],
+                "tcp",
+                "binary.example.org",
+                b"",
+                True,
+                id="allow+ignore: allowed but ignored",
+            ),
         ],
     )
     def test_ignore_connection(
@@ -233,24 +360,51 @@ class TestNextLayer:
                 tctx.configure(nl, ignore_hosts=ignore)
             if allow:
                 tctx.configure(nl, allow_hosts=allow)
-
             ctx = Context(
-                Client(peername=("192.168.0.42", 51234), sockname=("0.0.0.0", 8080)),
+                Client(
+                    peername=("192.168.0.42", 51234),
+                    sockname=("0.0.0.0", 8080),
+                    sni="existing-sni.example",
+                ),
                 tctx.options,
             )
             ctx.client.transport_protocol = transport_protocol
             if server_address:
                 ctx.server.address = (server_address, 443)
-                ctx.server.peername = ("1.2.3.4", 443)
-
+                ctx.server.peername = (
+                    ("2001:db8::1", 443, 0, 0)
+                    if server_address.startswith("ipv6")
+                    else ("192.0.2.1", 443)
+                )
             if result is NeedsMoreData:
                 with pytest.raises(NeedsMoreData):
-                    nl._ignore_connection(ctx, data_client)
+                    nl._ignore_connection(ctx, data_client, b"")
             else:
-                assert nl._ignore_connection(ctx, data_client) is result
+                assert nl._ignore_connection(ctx, data_client, b"") is result
+
+    def test_show_ignored_hosts(self, monkeypatch):
+        nl = NextLayer()
+
+        with taddons.context(nl) as tctx:
+            m = MagicMock()
+            m.context = Context(
+                Client(peername=("192.168.0.42", 51234), sockname=("0.0.0.0", 8080)),
+                tctx.options,
+            )
+            m.context.layers = [modes.TransparentProxy(m.context)]
+            m.context.server.address = ("example.com", 42)
+            tctx.configure(nl, ignore_hosts=["example.com"])
+
+            # Connection is ignored (not-MITM'ed)
+            assert nl._ignore_connection(m.context, http_get, b"") is True
+            # No flow is being set (i.e. nothing shown in UI)
+            assert nl._next_layer(m.context, http_get, b"").flow is None
+            # ... until `--show-ignored-hosts` is set:
+            tctx.configure(nl, show_ignored_hosts=True)
+            assert nl._next_layer(m.context, http_get, b"").flow is not None
 
     def test_next_layer(self, monkeypatch, caplog):
-        caplog.set_level(logging.INFO)
+        caplog.set_level(logging.DEBUG)
         nl = NextLayer()
 
         with taddons.context(nl) as tctx:
@@ -268,6 +422,7 @@ class TestNextLayer:
             assert m.layer is preexisting
 
             m.layer = None
+            monkeypatch.setattr(m, "data_client", lambda: http_get)
             nl.next_layer(m)
             assert m.layer
 
@@ -281,29 +436,51 @@ class TestNextLayer:
 
 
 @dataclass
-class TestConf:
+class TConf:
     before: list[type[Layer]]
     after: list[type[Layer]]
     proxy_mode: str = "regular"
     transport_protocol: TransportProtocol = "tcp"
+    tls_version: TlsVersion = None
     data_client: bytes = b""
     data_server: bytes = b""
     ignore_hosts: Sequence[str] = ()
     tcp_hosts: Sequence[str] = ()
     udp_hosts: Sequence[str] = ()
     ignore_conn: bool = False
+    server_address: Address | None = None
+    alpn: bytes | None = None
 
 
 explicit_proxy_configs = [
     pytest.param(
-        TestConf(
+        TConf(
             before=[modes.HttpProxy],
             after=[modes.HttpProxy, HttpLayer],
+            data_client=http_connect,
         ),
-        id=f"explicit proxy: regular http",
+        id=f"explicit proxy: regular http connect",
     ),
     pytest.param(
-        TestConf(
+        TConf(
+            before=[modes.HttpProxy],
+            after=[modes.HttpProxy, HttpLayer],
+            ignore_hosts=[".+"],
+            data_client=http_connect,
+        ),
+        id=f"explicit proxy: regular http connect disregards ignore_hosts",
+    ),
+    pytest.param(
+        TConf(
+            before=[modes.HttpProxy],
+            after=[modes.HttpProxy, HttpLayer],
+            ignore_hosts=[".+"],
+            data_client=http_get_absolute,
+        ),
+        id=f"explicit proxy: HTTP over regular proxy disregards ignore_hosts",
+    ),
+    pytest.param(
+        TConf(
             before=[modes.HttpProxy],
             after=[modes.HttpProxy, ClientTLSLayer, HttpLayer],
             data_client=client_hello_no_extensions,
@@ -311,14 +488,31 @@ explicit_proxy_configs = [
         id=f"explicit proxy: secure web proxy",
     ),
     pytest.param(
-        TestConf(
+        TConf(
+            before=[
+                modes.HttpProxy,
+                ClientTLSLayer,
+                partial(HttpLayer, mode=HTTPMode.regular),
+                partial(HttpStream, stream_id=1),
+            ],
+            after=[modes.HttpProxy, ClientTLSLayer, HttpLayer, HttpStream, TCPLayer],
+            server_address=("192.0.2.1", 443),
+            ignore_hosts=[".+"],
+            ignore_conn=True,
+            data_client=client_hello_with_extensions,
+            alpn=b"http/1.1",
+        ),
+        id=f"explicit proxy: ignore_hosts over established secure web proxy",
+    ),
+    pytest.param(
+        TConf(
             before=[modes.HttpUpstreamProxy],
             after=[modes.HttpUpstreamProxy, HttpLayer],
         ),
         id=f"explicit proxy: upstream proxy",
     ),
     pytest.param(
-        TestConf(
+        TConf(
             before=[modes.HttpUpstreamProxy],
             after=[modes.HttpUpstreamProxy, ClientQuicLayer, HttpLayer],
             transport_protocol="udp",
@@ -326,7 +520,7 @@ explicit_proxy_configs = [
         id=f"explicit proxy: experimental http3",
     ),
     pytest.param(
-        TestConf(
+        TConf(
             before=[
                 modes.HttpProxy,
                 partial(HttpLayer, mode=HTTPMode.regular),
@@ -338,7 +532,7 @@ explicit_proxy_configs = [
         id=f"explicit proxy: HTTP over regular proxy",
     ),
     pytest.param(
-        TestConf(
+        TConf(
             before=[
                 modes.HttpProxy,
                 partial(HttpLayer, mode=HTTPMode.regular),
@@ -356,7 +550,7 @@ explicit_proxy_configs = [
         id=f"explicit proxy: TLS over regular proxy",
     ),
     pytest.param(
-        TestConf(
+        TConf(
             before=[
                 modes.HttpProxy,
                 partial(HttpLayer, mode=HTTPMode.regular),
@@ -377,14 +571,14 @@ explicit_proxy_configs = [
         id=f"explicit proxy: HTTPS over regular proxy",
     ),
     pytest.param(
-        TestConf(
+        TConf(
             before=[
                 modes.HttpProxy,
                 partial(HttpLayer, mode=HTTPMode.regular),
                 partial(HttpStream, stream_id=1),
             ],
             after=[modes.HttpProxy, HttpLayer, HttpStream, TCPLayer],
-            data_client=b"\xFF",
+            data_client=b"\xff",
         ),
         id=f"explicit proxy: TCP over regular proxy",
     ),
@@ -404,7 +598,7 @@ for proto_plain, proto_enc, app_layer in [
     reverse_proxy_configs.extend(
         [
             pytest.param(
-                TestConf(
+                TConf(
                     before=[modes.ReverseProxy],
                     after=[modes.ReverseProxy, app_layer],
                     proxy_mode=f"reverse:{proto_plain}://example.com:42",
@@ -412,7 +606,7 @@ for proto_plain, proto_enc, app_layer in [
                 id=f"reverse proxy: {proto_plain} -> {proto_plain}",
             ),
             pytest.param(
-                TestConf(
+                TConf(
                     before=[modes.ReverseProxy],
                     after=[
                         modes.ReverseProxy,
@@ -426,7 +620,7 @@ for proto_plain, proto_enc, app_layer in [
                 id=f"reverse proxy: {proto_enc} -> {proto_enc}",
             ),
             pytest.param(
-                TestConf(
+                TConf(
                     before=[modes.ReverseProxy],
                     after=[modes.ReverseProxy, ClientTLSLayer, app_layer],
                     proxy_mode=f"reverse:{proto_plain}://example.com:42",
@@ -435,7 +629,7 @@ for proto_plain, proto_enc, app_layer in [
                 id=f"reverse proxy: {proto_enc} -> {proto_plain}",
             ),
             pytest.param(
-                TestConf(
+                TConf(
                     before=[modes.ReverseProxy],
                     after=[modes.ReverseProxy, ServerTLSLayer, app_layer],
                     proxy_mode=f"reverse:{proto_enc}://example.com:42",
@@ -448,7 +642,7 @@ for proto_plain, proto_enc, app_layer in [
 reverse_proxy_configs.extend(
     [
         pytest.param(
-            TestConf(
+            TConf(
                 before=[modes.ReverseProxy],
                 after=[modes.ReverseProxy, DNSLayer],
                 proxy_mode="reverse:dns://example.com:53",
@@ -456,7 +650,7 @@ reverse_proxy_configs.extend(
             id="reverse proxy: dns",
         ),
         pytest.param(
-            TestConf(
+            http3 := TConf(
                 before=[modes.ReverseProxy],
                 after=[modes.ReverseProxy, ServerQuicLayer, ClientQuicLayer, HttpLayer],
                 proxy_mode="reverse:http3://example.com",
@@ -464,7 +658,15 @@ reverse_proxy_configs.extend(
             id="reverse proxy: http3",
         ),
         pytest.param(
-            TestConf(
+            dataclasses.replace(
+                http3,
+                proxy_mode="reverse:https://example.com",
+                transport_protocol="udp",
+            ),
+            id="reverse proxy: http3 in https mode",
+        ),
+        pytest.param(
+            TConf(
                 before=[modes.ReverseProxy],
                 after=[
                     modes.ReverseProxy,
@@ -476,12 +678,24 @@ reverse_proxy_configs.extend(
             ),
             id="reverse proxy: quic",
         ),
+        pytest.param(
+            TConf(
+                before=[modes.ReverseProxy],
+                after=[modes.ReverseProxy, TCPLayer],
+                proxy_mode=f"reverse:http://example.com",
+                ignore_hosts=["example.com"],
+                server_address=("example.com", 80),
+                data_client=http_get,
+                ignore_conn=True,
+            ),
+            id="reverse proxy: ignore_hosts",
+        ),
     ]
 )
 
 transparent_proxy_configs = [
     pytest.param(
-        TestConf(
+        TConf(
             before=[modes.TransparentProxy],
             after=[modes.TransparentProxy, ServerTLSLayer, ClientTLSLayer],
             data_client=client_hello_no_extensions,
@@ -489,7 +703,7 @@ transparent_proxy_configs = [
         id=f"transparent proxy: tls",
     ),
     pytest.param(
-        TestConf(
+        TConf(
             before=[modes.TransparentProxy],
             after=[modes.TransparentProxy, ServerTLSLayer, ClientTLSLayer],
             data_client=dtls_client_hello_with_extensions,
@@ -498,16 +712,24 @@ transparent_proxy_configs = [
         id=f"transparent proxy: dtls",
     ),
     pytest.param(
-        TestConf(
+        quic := TConf(
             before=[modes.TransparentProxy],
             after=[modes.TransparentProxy, ServerQuicLayer, ClientQuicLayer],
             data_client=quic_client_hello,
             transport_protocol="udp",
+            server_address=("192.0.2.1", 443),
         ),
         id="transparent proxy: quic",
     ),
     pytest.param(
-        TestConf(
+        dataclasses.replace(
+            quic,
+            data_client=quic_short_header_packet,
+        ),
+        id="transparent proxy: existing quic session",
+    ),
+    pytest.param(
+        TConf(
             before=[modes.TransparentProxy],
             after=[modes.TransparentProxy, TCPLayer],
             data_server=b"220 service ready",
@@ -515,17 +737,36 @@ transparent_proxy_configs = [
         id="transparent proxy: raw tcp",
     ),
     pytest.param(
-        http := TestConf(
+        http := TConf(
             before=[modes.TransparentProxy],
             after=[modes.TransparentProxy, HttpLayer],
-            data_client=b"GET / HTTP/1.1\r\n",
+            server_address=("192.0.2.1", 80),
+            data_client=http_get,
         ),
         id="transparent proxy: http",
     ),
+    pytest.param(
+        TConf(
+            before=[modes.TransparentProxy, ServerTLSLayer, ClientTLSLayer],
+            after=[modes.TransparentProxy, ServerTLSLayer, ClientTLSLayer, HttpLayer],
+            data_client=b"GO /method-too-short-for-heuristic HTTP/1.1\r\n",
+            alpn=HTTP1_ALPNS[0],
+        ),
+        id=f"transparent proxy: http via ALPN",
+    ),
+    pytest.param(
+        TConf(
+            before=[modes.TransparentProxy],
+            after=[modes.TransparentProxy, TCPLayer],
+            server_address=("192.0.2.1", 23),
+            data_client=b"SSH-2.0-OpenSSH_9.7",
+        ),
+        id="transparent proxy: ssh",
+    ),
     pytest.param(
         dataclasses.replace(
             http,
-            tcp_hosts=["example.com"],
+            tcp_hosts=["192.0.2.1"],
             after=[modes.TransparentProxy, TCPLayer],
         ),
         id="transparent proxy: tcp_hosts",
@@ -533,38 +774,81 @@ transparent_proxy_configs = [
     pytest.param(
         dataclasses.replace(
             http,
-            ignore_hosts=["example.com"],
+            ignore_hosts=["192.0.2.1"],
             after=[modes.TransparentProxy, TCPLayer],
             ignore_conn=True,
         ),
         id="transparent proxy: ignore_hosts",
     ),
     pytest.param(
-        dns := TestConf(
+        TConf(
             before=[modes.TransparentProxy],
-            after=[modes.TransparentProxy, DNSLayer],
-            transport_protocol="udp",
-            data_client=dns_query,
+            after=[modes.TransparentProxy, TCPLayer],
+            data_client=custom_base64_proto,
         ),
-        id="transparent proxy: dns",
+        id="transparent proxy: full alpha tcp",
     ),
     pytest.param(
-        TestConf(
+        udp := TConf(
             before=[modes.TransparentProxy],
             after=[modes.TransparentProxy, UDPLayer],
+            server_address=("192.0.2.1", 553),
             transport_protocol="udp",
-            data_client=b"\xFF",
+            data_client=b"\xff",
         ),
         id="transparent proxy: raw udp",
     ),
+    pytest.param(
+        dns := dataclasses.replace(
+            udp,
+            after=[modes.TransparentProxy, DNSLayer],
+            data_client=dns_query,
+            server_address=("192.0.2.1", 53),
+        ),
+        id="transparent proxy: dns over udp",
+    ),
     pytest.param(
         dataclasses.replace(
             dns,
-            udp_hosts=["example.com"],
+            transport_protocol="tcp",
+        ),
+        id="transparent proxy: dns over tcp",
+    ),
+    pytest.param(
+        dataclasses.replace(
+            udp,
+            udp_hosts=["192.0.2.1"],
             after=[modes.TransparentProxy, UDPLayer],
         ),
         id="transparent proxy: udp_hosts",
     ),
+    pytest.param(
+        TConf(
+            before=[modes.TransparentProxy],
+            after=[modes.TransparentProxy, DNSLayer],
+            proxy_mode="wireguard",
+            server_address=("10.0.0.53", 53),
+            ignore_hosts=[".+"],
+            transport_protocol="udp",
+            data_client=dns_query,
+        ),
+        id="wireguard proxy: dns should not be ignored",
+    ),
+    pytest.param(
+        TConf(
+            before=[modes.TransparentProxy, ServerQuicLayer, ClientQuicLayer],
+            after=[
+                modes.TransparentProxy,
+                ServerQuicLayer,
+                ClientQuicLayer,
+                RawQuicLayer,
+            ],
+            data_client=b"<insert valid quic here>",
+            alpn=b"doq",
+            tls_version="QUICv1",
+        ),
+        id=f"transparent proxy: non-http quic",
+    ),
 ]
 
 
@@ -577,7 +861,7 @@ transparent_proxy_configs = [
     ],
 )
 def test_next_layer(
-    test_conf: TestConf,
+    test_conf: TConf,
 ):
     nl = NextLayer()
     with taddons.context(nl) as tctx:
@@ -589,12 +873,16 @@ def test_next_layer(
         )
 
         ctx = Context(
-            Client(peername=("192.168.0.42", 51234), sockname=("0.0.0.0", 8080)),
+            Client(
+                peername=("192.168.0.42", 51234),
+                sockname=("0.0.0.0", 8080),
+                alpn=test_conf.alpn,
+            ),
             tctx.options,
         )
-        ctx.server.address = ("example.com", 42)
-        # these aren't properly set up, but this does not matter here.
+        ctx.server.address = test_conf.server_address
         ctx.client.transport_protocol = test_conf.transport_protocol
+        ctx.client.tls_version = test_conf.tls_version
         ctx.client.proxy_mode = ProxyMode.parse(test_conf.proxy_mode)
         ctx.layers = [x(ctx) for x in test_conf.before]
         nl._next_layer(
@@ -602,8 +890,24 @@ def test_next_layer(
             data_client=test_conf.data_client,
             data_server=test_conf.data_server,
         )
-        assert stack_match(ctx, test_conf.after)
+        assert stack_match(ctx, test_conf.after), f"Unexpected stack: {ctx.layers}"
 
         last_layer = ctx.layers[-1]
         if isinstance(last_layer, (UDPLayer, TCPLayer)):
             assert bool(last_layer.flow) ^ test_conf.ignore_conn
+
+
+def test_starts_like_quic():
+    assert not _starts_like_quic(b"", ("192.0.2.1", 443))
+    assert not _starts_like_quic(dtls_client_hello_with_extensions, ("192.0.2.1", 443))
+
+    # Long Header - we can get definite answers from version numbers.
+    assert _starts_like_quic(quic_client_hello, None)
+    quic_version_negotation_grease = bytes.fromhex(
+        "ca0a0a0a0a08c0618c84b54541320823fcce946c38d8210044e6a93bbb283593f75ffb6f2696b16cfdcb5b1255"
+    )
+    assert _starts_like_quic(quic_version_negotation_grease, None)
+
+    # Short Header - port-based is the best we can do.
+    assert _starts_like_quic(quic_short_header_packet, ("192.0.2.1", 443))
+    assert not _starts_like_quic(quic_short_header_packet, ("192.0.2.1", 444))
