@@ -8,6 +8,5 @@ shutil.copy(src / "9da13359.0", "9da13359.0")
 
 for x in ["self-signed", "trusted-leaf", "trusted-root"]:
     (here / f"{x}.pem").write_text(
-        (src / f"{x}.crt").read_text() +
-        (src / f"{x}.key").read_text()
+        (src / f"{x}.crt").read_text() + (src / f"{x}.key").read_text()
     )
