@@ -1,4 +1,5 @@
 import io
+from pathlib import Path
 
 import pytest
 from hypothesis import example
@@ -10,11 +11,14 @@ from mitmproxy import version
 from mitmproxy.io import FlowReader
 from mitmproxy.io import tnetstring
 
+here = Path(__file__).parent.parent / "data"
+
 
 class TestFlowReader:
     @given(binary())
     @example(b"51:11:12345678901#4:this,8:true!0:~,4:true!0:]4:\\x00,~}")
     @example(b"0:")
+    @example(b"0:~")
     def test_fuzz(self, data):
         f = io.BytesIO(data)
         reader = FlowReader(f)
@@ -24,6 +28,18 @@ class TestFlowReader:
         except exceptions.FlowReadException:
             pass  # should never raise anything else.
 
+    @pytest.mark.parametrize(
+        "file", [pytest.param(x, id=x.stem) for x in here.glob("har_files/*.har")]
+    )
+    def test_har(self, file):
+        with open(file, "rb") as f:
+            reader = FlowReader(f)
+            try:
+                for _ in reader.stream():
+                    pass
+            except exceptions.FlowReadException:
+                pass  # should never raise anything else.
+
     def test_empty(self):
         assert list(FlowReader(io.BytesIO(b"")).stream()) == []
 
