@@ -55,6 +55,7 @@ def test_bytes_to_escaped_str():
     assert strutils.bytes_to_escaped_str(b"\r\n\t") == "\\r\\n\\t"
     assert strutils.bytes_to_escaped_str(b"\r\n\t", True) == "\r\n\t"
 
+    assert strutils.bytes_to_escaped_str(b"\n", False) == r"\n"
     assert strutils.bytes_to_escaped_str(b"\n", True) == "\n"
     assert strutils.bytes_to_escaped_str(b"\\n", True) == "\\ \\ n".replace(" ", "")
     assert strutils.bytes_to_escaped_str(b"\\\n", True) == "\\ \\ \n".replace(" ", "")
@@ -79,9 +80,20 @@ def test_escaped_str_to_bytes():
 
 
 def test_is_mostly_bin():
-    assert not strutils.is_mostly_bin(b"foo\xFF")
-    assert strutils.is_mostly_bin(b"foo" + b"\xFF" * 10)
-    assert not strutils.is_mostly_bin("")
+    assert not strutils.is_mostly_bin(b"foo\xff")
+    assert strutils.is_mostly_bin(b"foo" + b"\xff" * 10)
+    assert not strutils.is_mostly_bin(b"")
+    assert strutils.is_mostly_bin(b"\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09")
+    # shift UTF8 break point
+    # 𐍅 is four bytes in UTF-8, so we're breaking the 100 chars barrier.
+    assert not strutils.is_mostly_bin(b"" + 50 * "𐍅".encode())
+    assert not strutils.is_mostly_bin(b"a" + 50 * "𐍅".encode())
+    assert not strutils.is_mostly_bin(b"aa" + 50 * "𐍅".encode())
+    assert not strutils.is_mostly_bin(b"aaa" + 50 * "𐍅".encode())
+    assert not strutils.is_mostly_bin(b"aaaa" + 50 * "𐍅".encode())
+    assert not strutils.is_mostly_bin(b"aaaaa" + 50 * "𐍅".encode())
+    # only utf8 continuation chars
+    assert strutils.is_mostly_bin(150 * b"\x80")
 
 
 def test_is_xml():
@@ -137,3 +149,16 @@ def test_escape_special_areas():
     esc = strutils.escape_special_areas('foo "b*r" b*z', ESCAPE_QUOTES, "*")
     assert esc == 'foo "b\ue02ar" b*z'
     assert strutils.unescape_special_areas(esc) == 'foo "b*r" b*z'
+
+
+@pytest.mark.parametrize(
+    "content,n,expected",
+    [
+        ("foo\nbar\nbaz", 1, "foo\n"),
+        ("foo\nbar\nbaz", 2, "foo\nbar\n"),
+        ("foo\nbar", 100, "foo\nbar"),
+        ("\nbar", 1, "\n"),
+    ],
+)
+def test_cut_after_n_newlines(content, n, expected):
+    assert strutils.cut_after_n_lines(content, n) == expected
