@@ -1,21 +1,21 @@
 import * as React from "react";
 import ConnectionIndicator from "../../../components/Header/ConnectionIndicator";
 import * as connectionActions from "../../../ducks/connection";
-import { render } from "../../test-utils";
+import { act, render } from "../../test-utils";
+import { TStore } from "../../ducks/tutils";
 
 test("ConnectionIndicator", async () => {
-    const { asFragment, store } = render(<ConnectionIndicator />);
+    const { asFragment, store } = render(<ConnectionIndicator />, {
+        store: TStore(null),
+    });
     expect(asFragment()).toMatchSnapshot();
 
-    store.dispatch(connectionActions.startFetching());
+    act(() => store.dispatch(connectionActions.startFetching()));
     expect(asFragment()).toMatchSnapshot();
 
-    store.dispatch(connectionActions.connectionEstablished());
+    act(() => store.dispatch(connectionActions.finishFetching()));
     expect(asFragment()).toMatchSnapshot();
 
-    store.dispatch(connectionActions.connectionError("wat"));
-    expect(asFragment()).toMatchSnapshot();
-
-    store.dispatch(connectionActions.setOffline());
+    act(() => store.dispatch(connectionActions.connectionError("wat")));
     expect(asFragment()).toMatchSnapshot();
 });
