@@ -5,10 +5,13 @@
  * - read the initial URL state on page load
  * - push updates to the URL later on.
  */
-import { select, setFilter, setHighlight } from "./ducks/flows";
+import { FilterName, setFilter, setHighlight } from "./ducks/ui/filter";
+import { select } from "./ducks/flows";
 import { selectTab } from "./ducks/ui/flow";
 import * as eventLogActions from "./ducks/eventLog";
 import * as commandBarActions from "./ducks/commandBar";
+import { RootStore } from "./ducks/store";
+import { Tab, setCurrent } from "./ducks/ui/tabs";
 
 const Query = {
     SEARCH: "s",
@@ -17,22 +20,33 @@ const Query = {
     SHOW_COMMANDBAR: "c",
 };
 
-export function updateStoreFromUrl(store) {
+export function updateStoreFromUrl(store: RootStore) {
     const [path, query] = window.location.hash.substr(1).split("?", 2);
     const path_components = path.substr(1).split("/");
 
     if (path_components[0] === "flows") {
         if (path_components.length == 3) {
             const [flowId, tab] = path_components.slice(1);
-            store.dispatch(select(flowId));
             store.dispatch(selectTab(tab));
+
+            const selectFlowOnceAvailable = () => {
+                const flow = store.getState().flows.byId.get(flowId);
+                if (flow !== undefined) {
+                    unsubscribe();
+                    store.dispatch(select([flow]));
+                }
+            };
+            const unsubscribe = store.subscribe(selectFlowOnceAvailable);
+            selectFlowOnceAvailable();
         }
+    } else if (path_components[0] === "capture") {
+        store.dispatch(setCurrent(Tab.Capture));
     }
 
     if (query) {
         query.split("&").forEach((x) => {
-            let [key, value] = x.split("=", 2);
-            value = decodeURIComponent(value);
+            const [key, encodedVal] = x.split("=", 2);
+            const value = decodeURIComponent(encodedVal);
             switch (key) {
                 case Query.SEARCH:
                     store.dispatch(setFilter(value));
@@ -55,22 +69,24 @@ export function updateStoreFromUrl(store) {
     }
 }
 
-export function updateUrlFromStore(store) {
+export function updateUrlFromStore(store: RootStore) {
     const state = store.getState();
-    let query = {
-        [Query.SEARCH]: state.flows.filter,
-        [Query.HIGHLIGHT]: state.flows.highlight,
+    const query = {
+        [Query.SEARCH]: state.ui.filter[FilterName.Search],
+        [Query.HIGHLIGHT]: state.ui.filter[FilterName.Highlight],
         [Query.SHOW_EVENTLOG]: state.eventLog.visible,
         [Query.SHOW_COMMANDBAR]: state.commandBar.visible,
     };
     const queryStr = Object.keys(query)
         .filter((k) => query[k])
-        .map((k) => `${k}=${encodeURIComponent(query[k])}`)
+        .map((k) => `${k}=${encodeURIComponent(query[k]!)}`)
         .join("&");
 
     let url;
-    if (state.flows.selected.length > 0) {
-        url = `/flows/${state.flows.selected[0]}/${state.ui.flow.tab}`;
+    if (state.ui.tabs.current === Tab.Capture) {
+        url = "/capture";
+    } else if (state.flows.selected.length > 0) {
+        url = `/flows/${state.flows.selected[0].id}/${state.ui.flow.tab}`;
     } else {
         url = "/flows";
     }
